#########################################
# Created 02/2020 Francesc Roura Adserias
#########################################
#-------------------------------------------------------------------
# Libraries & functions load
#-------------------------------------------------------------------
variable <- "tas"
fcst.sdate <- "202002"
skill <- TRUE
region <- TRUE
country <- FALSE
global <- TRUE
path.outside <- "/home/Earth/froura/"

cat("Variable:   "); print(variable) 
cat("Sdate: "); print(fcst.sdate)
cat("Skill: "); print(skill) 
cat("Country: "); print(country) 
cat("GLOBAL: "); print(global)
cat("Outpath: "); print(path.outside)

.libPaths(new = '/gpfs/projects/bsc32/share/R_libs/3.5') #goes to the packages locations

library(s2dverification) 
library(ncdf4)
library(abind)
library(easyVerification)
library(multiApply)
library(startR)
library(easyNCDF)
library(pryr) # To check mem usage.

setwd("/home/Earth/froura/repositories/S2S4E-backend-BSC/data-analysis/")
source("create_nc.R")  # Load function create_nc
source("create_nc_country.R")  # Load function create_nc
source("Calibration_fcst.R")
source("s2s4e_tools.R")
library(CSTools)

#-------------------------------------------------------------------
# Settings 
#-------------------------------------------------------------------

variable <- "tas" # for Decathlon we will only forecast temperature  
system.name <- "system5"
exp.dir <- "/esarchive/exp/ecmwf/system5c3s/" 
remap.method <- "con" 
hcst.inityear <- 2015 #1993 
hcst.endyear <- 2017 #2017
#ncores=16 #(?)
ncores = 1
split_factor=1 #(?)

lons.min <- 0  #0
lons.max <- 360  #360
lats.min <- -90  #-90
lats.max <- 90 # 90

#-------------------------------------------------------------------
# Init 
#-------------------------------------------------------------------

# frequencies of obs and exp for temperature (tas)

freq.exp <- "f6h"
freq.obs <- "f1h"

# Start dates and Lead times

fcst.month <- substr(fcst.sdate,5,6) 
fcst.year <- substr(fcst.sdate,1,4) 

fcst.sdate <- paste(fcst.year, fcst.month, sep = "")
hcst.sdates <- paste(strtoi(hcst.inityear):strtoi(hcst.endyear),
                     fcst.month, sep = "")

obs.sdates <- hcst.sdates

if (system.name == "system5") {
  hcst.nmember <- 25
} else {
  stop("The system is not system5") 
}   

ltmin <- 2
fcst.freq <- "monthly_mean"
ltmax <- 4

#-------------------------------------------------------------------
# Timeseries load 
#-------------------------------------------------------------------
obs.path <- paste0(
            "/esarchive/recon/ecmwf/era5/",fcst.freq,
            "/tas_",
            freq.obs,
            "/tas_$file_date$.nc"
            )

fcst.path <- paste0( 
             exp.dir,fcst.freq, "/tas_", freq.exp,
             "/tas_$sdate$01.nc"
             )

hcst.path <- paste0(
             exp.dir,fcst.freq,"/tas_",
             freq.exp,
             "/tas_$sdate$01.nc"
             )

#-------------------------------------------------------------------
# Load data
#-------------------------------------------------------------------

hcst <- Start(dat = hcst.path,
              var = variable,
              sdate = hcst.sdates,
              time = indices(ltmin:ltmax),
              latitude = values(list(lats.min, lats.max)),
              longitude = values(list(lons.min, lons.max)),
              synonims = list(latitude=c('lat','latitude'),
                              longitude=c('lon','longitude')), 
              ensemble = indices(1:hcst.nmember),
              return_vars = list(latitude = 'dat',
                                 longitude = 'dat',
                                 time = c('sdate')),
              retrieve = T)

fcst <- Start(dat = fcst.path, 
              var = variable,
              sdate = fcst.sdate,
              time = indices(ltmin:ltmax),
              latitude = values(list(lats.min, lats.max)),
              longitude = values(list(lons.min, lons.max)),
              synonims = list(latitude=c('lat','latitude'),
                              longitude=c('lon','longitude')), 
              ensemble= 'all',
              return_vars = list(latitude = 'dat',
                                 longitude = 'dat',
                                 time = c('sdate')),
              retrieve = T)

dates <- attr(hcst, 'Variables')$common$time
dates_file <- sapply(dates, format, '%Y%m')
dim(dates_file) <- dim(dates)

obs <- Start(dat = obs.path, # there's a dimension that is 1:51, and i don't know what is that
             var = variable,
             file_date = dates_file,
             latitude = values(list(lats.min, lats.max)),
             longitude = values(list(lons.min, lons.max)),
             synonims = list(latitude=c('lat','latitude'),
                             longitude=c('lon','longitude')), 
             transform = CDORemapper,
             transform_params = list(
               grid = paste0(
                 "/esarchive/exp/ecmwf/system5c3s/",fcst.freq,
                 "/",variable,"_",
                 freq.exp,
                 ## TODO: replace hardcoded grid sample
                 #"/",variable,"_",fcst.sdate,"01.nc"
                 "/",variable,"_","201805","01.nc"
               ),
               method = 'con',
               crop = c(lons.min,lons.max,lats.min,lats.max)
             ), 
             transform_vars = c('latitude', 'longitude'), 
             return_vars = list(latitude = 'dat',
                                longitude = 'dat',
                                time = 'file_date'),
             split_multiselected_dims = TRUE,
             retrieve = TRUE)

print("######### DATA LOADING COMPLETED ##############")

#-------------------------------------------------------------------
# seasonal aggregation DECATHLON
#-------------------------------------------------------------------
seasonal.agg=F ####### this chunk has to be passet to Apply #######
if(seasonal.agg == T){

  attr.hcst <- attributes(hcst) 
  attr.hcst$dim[4] <- 1
  hcst <- Mean1Dim(hcst,4)
  default_dims <- c(dat=1, var=1, sdate = 1, time=1, latitude=1, longitude=1, ensemble = 1)
  default_dims[names(dim(hcst))] <- dim(hcst)
  dim(hcst) <- default_dims
  attributes(hcst) <- attr.hcst
  
  attr.fcst <- attributes(fcst) 
  attr.fcst$dim[4] <- 1
  fcst <- Mean1Dim(fcst,4)
  default_dims <- c(dat=1, var=1, sdate = 1, time=1, latitude=1, longitude=1, ensemble = 1)
  default_dims[names(dim(fcst))] <- dim(fcst)
  dim(fcst) <- default_dims
  attributes(fcst) <- attr.fcst

  attr.obs <- attributes(obs) 
  attr.obs$dim[4] <- 1  
  obs <- Mean1Dim(obs,4)
  attributes(obs) <- attr.obs

# here the same is done but using Apply(), that maintains the attributes of the input.
#hcst.dummy <- Apply(data=list(hcst), use_attributes = list(names(dim(hcst))),
 #                         target_dims=list(names(dim(hcst))[4]),
  #                        ncores=ncores,
   #                       fun = mean)

#fcst <- Apply(data=list(var_obs=sub_obs,
 #                                   var_exp=sub_hcst,
  #                                  var_fcst=sub_fcst),
   #                       target_dims=c('time'),
    #                      output_dims=c('time'),
     #                     ncores=ncores,
      #                    fun = mean)[[1]]
}

#-------------------------------------------------------------------
# Forecast Bias adjustment (calibration method)
#-------------------------------------------------------------------

default_dims <- c(dat=1, var=1, sdate = 1, time=1, latitude=1, longitude=1, ensemble = 1)
default_dims[names(dim(obs))] <- dim(obs)
dim(obs) <- default_dims

sub_obs<-Subset(obs,c('dat','var'),list(1,1),drop='selected')
sub_hcst<-Subset(hcst,c('dat','var'),list(1,1),drop='selected')
sub_fcst<-Subset(fcst,c('dat','var'),list(1,1),drop='selected')

lon <- attr(obs, 'Variables')$dat1$longitude
lat <- attr(obs, 'Variables')$dat1$latitude

fcst.times <- attr(fcst, 'Variables')$common$time
fcst.times <- sort(unique(sapply(as.character(fcst.times),
                                 substr, 1, 10)))
fcst.times[which(substr(fcst.times, 9, 10) == '01')] <- as.character(as.Date(fcst.times[which(substr(fcst.times, 9, 10) == '01')], "%Y-%m-%d")-1)


hcst.times <- attr(hcst, 'Variables')$common$time
hcst.times <- sort(unique(sapply(as.character(hcst.times),
                                 substr, 1, 10)))
hcst.times[which(substr(hcst.times, 9, 10) == '01')] <- as.character(as.Date(hcst.times[which(substr(hcst.times, 9, 10) == '01')], "%Y-%m-%d")-1)

remove(obs,hcst,fcst) # obs (?) el suprimeixo però sequeix apareixent-me com a no null

## TODO: Add accum vars 
if (variable == 'prlr' || variable == 'rsds'){
  sub_obs[sub_obs < 0 ] <- 0
  sub_hcst[sub_hcst < 0 ] <- 0
}

################# variables out ##################

mask.path <- '/home/Earth/froura/repositories/auto-s2s/data-analysis/masks/mask_region_decathlon.Rdata'
# var.exp
# var.obs

################# from here goes to the general script (s2s4e.global.R) ###################







################# to try, I've copypasted the general code  ###################

if (global == TRUE){
  
  calibrated_fcst <-Apply(data=list(var_obs=sub_obs,
                                    var_exp=sub_hcst,
                                    var_fcst=sub_fcst),
                          target_dims=c('sdate','ensemble'),
                          output_dims=c('ensemble'),
                          ncores=ncores,
                          fun = Calibration_fcst)[[1]]
  
  
  print("######### GLOBAL: FCST CALIBRATION COMPLETED ##############")
  #-------------------------------------------------------------------
  # Hindcast terciles and skill computation
  # (Only if the terciles are not loaded from external files) 
  #-------------------------------------------------------------------
  accum.vars <- c('prlr', 'rsds')
  
  if (variable %in% accum.vars){
    na.rm <- TRUE
  } else {
    na.rm <- FALSE
  }
  
  if ( skill == TRUE){ #aquí tinc problemes (Error in apply(hcst, c(2), mean) : dim(X) must have a positive length)
    calibrated_hcst <- Apply(data=list(var_obs=sub_obs,
                                       var_exp=sub_hcst),
                             target_dims=c('sdate','ensemble'),
                             output_dims=c('ensemble', 'sdate'),
                             ncores=ncores,
                             fun = Calibration)[[1]]
    
    
    if (variable == 'prlr' || variable == 'rsds'){
      calibrated_hcst[calibrated_hcst < 0 ] <- 0
    }
    calibrated_hcst[!is.finite(calibrated_hcst)] <- NA
    
    print("######### GLOBAL HCST CALIBRATION COMPLETED ##############")
    
    # Computing terciles and extremes from hindcast
    
    terciles_hcst<-Apply(calibrated_hcst,c('sdate','ensemble'),
                         quantile,c(1/3,2/3),output_dims="bin",ncores=ncores, na.rm=na.rm)[[1]]
    
    extremes_hcst<-Apply(calibrated_hcst,c('sdate','ensemble'),
                         quantile,c(1/10,9/10),output_dims="bin",ncores=ncores, na.rm=na.rm)[[1]]
    
    print("######### TERCILES COMPUTATION COMPLETED ##############")
    
    skill.global <- Compute_skill(calibrated_hcst, sub_obs, ncores=ncores)
    fRPSS <- skill.global$fRPSS
    BSS10 <- skill.global$BSS10
    BSS90 <- skill.global$BSS90
    
    remove(skill.global)
    
    
    print("######### SKILL COMPUTATION COMPLETED ##############")
    
    hcst.probs <- Compute_probs(calibrated_hcst, terciles_hcst, extremes_hcst, na.rm=na.rm, ncores=ncores)
    tercile_probs_hcst <- hcst.probs$tercile_probs_hcst
    extreme_probs_hcst <- hcst.probs$extreme_probs_hcst
    print("######### GLOBAL FCST PROBS COMPUTATION DONE ##############")
    
    
    
    print("######### FCST PROBS COMPUTATION DONE ##############")
    
    outfile = paste0(path.outside, variable,"-skill_month",fcst.month,".nc")
    save_skill_file(variable,
                    fRPSS,
                    BSS10,
                    BSS90,
                    fcst.sdate, 
                    lon,
                    lat,
                    outfile,
                    hcst.times)
    
    remove(fRPSS,BSS10,BSS90) 
    
    outfile = paste0(path.outside, variable,"-percentiles_month",fcst.month,".nc")
    save_terciles_file(variable,
                       terciles_hcst,
                       extremes_hcst,
                       fcst.sdate, 
                       lon,
                       lat,
                       outfile,
                       hcst.times)
    
    for (n in 1:((hcst.endyear-hcst.inityear)+1)) {
      outfile = paste0(path.outside, variable,"-prob_",hcst.sdates[n],"01_",fcst.month,".nc")
      save_probs_file(variable,
                      tercile_probs_hcst[,n,,,],
                      extreme_probs_hcst[,n,,,],
                      hcst.sdates[n],
                      lon,
                      lat,
                      outfile,
                      hcst.times[(1+((ltmax-ltmin)+1)*(n-1)):(((ltmax-ltmin)+1)*n)])
      
      outfile = paste0(path.outside, variable,"_",hcst.sdates[n],"01_",fcst.month,".nc")
      save_ensemble_file(variable,
                         calibrated_hcst[,n,,,],
                         hcst.sdates[n],
                         lon,
                         lat,
                         outfile,
                         hcst.times[(1+((ltmax-ltmin)+1)*(n-1)):(((ltmax-ltmin)+1)*n)])
    } 
    
    remove(calibrated_hcst)
    
  } else {
    
    ## TODO: Change hardcoded system
    fcst.month <- substr(fcst.sdate, start=5, stop=6)
    terciles.filepath <- paste0('/esarchive/oper/S2S4E-data/monthly_statistics/',
                                variable,'/', variable, '-percentiles_month', fcst.month, 
                                '.nc') 
    
    terciles_hcst <- NcToArray(terciles.filepath, vars_to_read=c('p33', 'p66'))
    extremes_hcst <- NcToArray(terciles.filepath, vars_to_read=c('p10', 'p90'))
    
    terciles_hcst <- aperm(terciles_hcst, c(1, 4, 2, 3))
    names(dim(terciles_hcst)) <- c('bin', 'time','latitude', 'longitude')
    
    extremes_hcst <- aperm(extremes_hcst, c(1, 4, 2, 3))
    names(dim(extremes_hcst)) <- c('bin', 'time','latitude', 'longitude')
    
    if (variable == "prlr") {
      for (month in 1:(dim(terciles_hcst)[2])){
        month.sdate <- paste0(substr(hcst.times[month],1,7), "-01")
        month.lastday <- as.integer(substr((seq(as.Date(month.sdate, "%Y-%m-%d"),
                                                length=2,by="months")-1)[2], 9, 10))
        terciles_hcst[,month,,] <- terciles_hcst[,month,,]/(month.lastday*86400000)
        extremes_hcst[,month,,] <- extremes_hcst[,month,,]/(month.lastday*86400000)
      }
    } else if (variable == "tasmin" || variable == "tasmax" || variable == "tas") {
      extremes_hcst <- extremes_hcst + 273
      terciles_hcst <- terciles_hcst + 273
    } else if (variable == "psl") {
      terciles_hcst <- terciles_hcst * 100
      extremes_hcst <- extremes_hcst * 100
    } else {
      warning("Unknown Variable")
    }
    
    
    print("######### TERCILES LOADED ##############")
    
    
  }
  
  
  #-------------------------------------------------------------------
  # Probabilities for each category
  # and extreme values
  #-------------------------------------------------------------------
  
  # Computing forecast probabilities
  
  fcst.probs <- Compute_probs(calibrated_fcst, terciles_hcst, extremes_hcst, na.rm=TRUE, ncores=ncores)
  tercile_probs <- fcst.probs$tercile_probs_hcst
  extreme_probs <- fcst.probs$extreme_probs_hcst
  print("######### GLOBAL FCST PROBS COMPUTATION DONE ##############")
  
  print("######### FCST PROBS COMPUTATION DONE ##############")
  
  #----------------------------------------------------------
  # Saving calibrated forecast like .nc
  #----------------------------------------------------------
  
  ## TODO: sub-seasonal
  outfile = paste0(path.outside, variable,"-prob_",fcst.sdate,"01_",fcst.month,".nc")
  save_probs_file(variable,
                  tercile_probs,
                  extreme_probs,
                  fcst.sdate,
                  lon,
                  lat,
                  outfile,
                  fcst.times) 
  
  outfile = paste0(path.outside, variable,"_",fcst.sdate,"01_",fcst.month,".nc")
  save_ensemble_file(variable,
                     calibrated_fcst,
                     fcst.sdate,
                     lon,
                     lat,
                     outfile,
                     fcst.times) 
  
  remove(fcst.probs, tercile_probs, extreme_probs)
  
}
region=F
if (country == TRUE | region == TRUE){
  if(country == TRUE){
    print("COUNTRY-AGGREGATED CALCULATION")
  } 
  if(region == TRUE) {
    print("REGION-AGGREGATED CALCULATION")
  } 
  #-------------------------------------------------------------------
  # Fcst, Hcst & Obs country-mean computation 
  #-------------------------------------------------------------------
  
  if(country == TRUE){
    mask.path <- '/esarchive/oper/S2S4E-backend-BSC/data-analysis/masks/mask_europe_system5_2.Rdata'
  } 
  if(region == TRUE){
    mask.path <- '/home/Earth/froura/repositories/auto-s2s/data-analysis/masks/mask_region_decathlon.Rdata'
  }
  
  
  load(mask.path)
  
  sub_hcst.country <-Apply(data=list(pointdata=sub_hcst),
                           target_dims=c('latitude','longitude'),
                           output_dims=c('country'),
                           mask.path=mask.path,
                           ncores=ncores,
                           split_factor=split_factor,
                           fun = Country_mean)[[1]]
  
  sub_fcst.country <-Apply(data=list(pointdata=sub_fcst),
                           target_dims=c('latitude','longitude'),
                           output_dims=c('country'),
                           mask.path=mask.path,
                           ncores=ncores,
                           split_factor=split_factor,
                           fun = Country_mean)[[1]]
  
  sub_obs.country <-Apply(data=list(pointdata=sub_obs),
                          target_dims=c('latitude','longitude'),
                          output_dims=c('country'),
                          mask.path=mask.path,
                          ncores=ncores,
                          split_factor=split_factor,
                          fun = Country_mean)[[1]]
  
  calibrated_fcst.country <-Apply(data=list(var_obs=sub_obs.country,
                                            var_exp=sub_hcst.country,
                                            var_fcst=sub_fcst.country),
                                  target_dims=c('sdate','ensemble'),
                                  output_dims=c('ensemble'),
                                  ncores=ncores,
                                  split_factor=split_factor,
                                  fun = Calibration_fcst)[[1]]
  
  
  
  print("######### COUNTRY, REGION: FCST CALIBRATION COMPLETED ##############")
  #-------------------------------------------------------------------
  # Hindcast terciles and skill computation
  # (Only if the terciles are not loaded from external files) 
  #-------------------------------------------------------------------
  accum.vars <- c('prlr', 'rsds')
  
  if (variable %in% accum.vars){
    na.rm <- TRUE
  } else {
    na.rm <- FALSE
  }
  
  if ( skill == TRUE){
    calibrated_hcst.country <-Apply(data=list(var_obs=sub_obs.country,
                                              var_exp=sub_hcst.country),
                                    target_dims=c('sdate','ensemble'),
                                    output_dims=c('ensemble', 'sdate'),
                                    ncores=ncores,
                                    fun = Calibration)[[1]]
    
    
    if (variable == 'prlr' || variable == 'rsds'){
      calibrated_hcst.country[calibrated_hcst.country < 0 ] <- 0
    }
    
    calibrated_hcst.country[!is.finite(calibrated_hcst.country)] <- NA
    
    print("######### COUNTRY HCST CALIBRATION COMPLETED ##############")
    # Computing terciles and extremes from hindcast
    
    terciles_hcst.country <- Apply(calibrated_hcst.country,c('sdate','ensemble'),
                                   quantile,c(1/3,2/3),output_dims="bin",ncores=ncores, na.rm=na.rm, split_factor=split_factor)[[1]]
    
    extremes_hcst.country <- Apply(calibrated_hcst.country,c('sdate','ensemble'),
                                   quantile,c(1/10,9/10),output_dims="bin",ncores=ncores, na.rm=na.rm, split_factor=split_factor)[[1]]
    
    
    print("######### TERCILES COMPUTATION COMPLETED ##############")
    
    skill.country <- Compute_skill(calibrated_hcst.country, sub_obs.country, ncores=ncores, na.rm=T)
    fRPSS.country <- skill.country$fRPSS
    BSS10.country <- skill.country$BSS10
    BSS90.country <- skill.country$BSS90
    
    print("######### COUNTRY SKILL COMPUTATION COMPLETED ##############")
    
    hcst.probs.country <- Compute_probs(calibrated_hcst.country, terciles_hcst.country, extremes_hcst.country,
                                        na.rm = TRUE)
    tercile_probs_hcst.country <- hcst.probs.country$tercile_probs_hcst
    extreme_probs_hcst.country <- hcst.probs.country$extreme_probs_hcst
    print("######### COUNTRY FCST PROBS COMPUTATION DONE ##############")
    
    
    print("######### COUNTRY FCST PROBS COMPUTATION DONE ##############")
    
    outfile = paste0(path.outside,"/../",variable,"_country/", variable,"-country-skill_month",fcst.month,".nc")
    save_skill_country_file(variable,
                            fRPSS.country,
                            BSS10.country,
                            BSS90.country,
                            fcst.sdate,
                            outfile,
                            hcst.times,
                            europe.countries.iso,
                            "seasonal")
    
    outfile = paste0(path.outside,"/../",variable,"_country/", variable,"-country-percentiles_month",fcst.month,".nc")
    save_terciles_country_file(variable,
                               terciles_hcst.country,
                               extremes_hcst.country,
                               fcst.sdate,
                               outfile,
                               hcst.times,
                               europe.countries.iso,
                               "seasonal")
    
    
    for (n in 1:((hcst.endyear-hcst.inityear)+1)) {
      outfile = paste0(path.outside,"/../",variable,
                       "_country/", variable,"-country-prob_",hcst.sdates[n],"01_",fcst.month,".nc")
      save_probs_country_file(variable,
                              tercile_probs_hcst.country[,n,,],
                              extreme_probs_hcst.country[,n,,],
                              hcst.sdates[n],
                              outfile,
                              hcst.times[(1+((ltmax-ltmin)+1)*(n-1)):(((ltmax-ltmin)+1)*n)],
                              europe.countries.iso,
                              "seasonal")
      
      outfile = paste0(path.outside,"/../",variable,
                       "_country/", variable,"-country_",hcst.sdates[n],"01_", fcst.month,".nc")
      save_ensemble_country_file(variable,
                                 aperm(calibrated_hcst.country[,n,,], c(2,1,3)),
                                 hcst.sdates[n],
                                 outfile,
                                 hcst.times[(1+((ltmax-ltmin)+1)*(n-1)):(((ltmax-ltmin)+1)*n)],
                                 europe.countries.iso,
                                 "seasonal")
      
    } 
  } else {
    
    ## TODO: Change hardcoded system
    fcst.month <- substr(fcst.sdate, start=5, stop=6)
    terciles.country.filepath <- paste0(path.outside, '/../',
                                        variable,'_country/', variable, '-country-percentiles_month', fcst.month, 
                                        '.nc') 
    
    terciles_hcst.country <- NcToArray(terciles.country.filepath, vars_to_read=c('p33', 'p66'))
    extremes_hcst.country <- NcToArray(terciles.country.filepath, vars_to_read=c('p10', 'p90'))
    
    terciles_hcst.country <- aperm(terciles_hcst.country, c(1, 3, 2))
    names(dim(terciles_hcst.country)) <- c('bin', 'time','country')
    
    extremes_hcst.country <- aperm(extremes_hcst.country, c(1, 3, 2))
    names(dim(extremes_hcst.country)) <- c('bin', 'time','country')
    
    if (variable == "prlr") {
      for (month in 1:(dim(terciles_hcst.country)[2])){
        month.sdate <- paste0(substr(hcst.times[month],1,7), "-01")
        month.lastday <- as.integer(substr((seq(as.Date(month.sdate, "%Y-%m-%d"),
                                                length=2,by="months")-1)[2], 9, 10))
        terciles_hcst.country[,month,] <- terciles_hcst.country[,month,]/(month.lastday*86400000)
        extremes_hcst.country[,month,] <- extremes_hcst.country[,month,]/(month.lastday*86400000)
      }
    } else if (variable == "tasmin" || variable == "tasmax" || variable == "tas") {
      extremes_hcst.country <- extremes_hcst.country + 273
      terciles_hcst.country <- terciles_hcst.country + 273
    } else if (variable == "psl") {
      terciles_hcst.country <- terciles_hcst.country * 100
      extremes_hcst.country <- extremes_hcst.country * 100
    } else {
      warning("Unknown Variable")
    }
    
    
    print("######### COUNTRY TERCILES LOADED ##############")
    
    
  }
  
  
  #-------------------------------------------------------------------
  # Probabilities for each category
  # and extreme values
  #-------------------------------------------------------------------
  
  # Computing forecast probabilities
  
  fcst.probs.country <- Compute_probs(calibrated_fcst.country, terciles_hcst.country,
                                      extremes_hcst.country, na.rm=TRUE, ncores=ncores)
  tercile_probs_fcst.country <- fcst.probs.country$tercile_probs_hcst
  extreme_probs_fcst.country <- fcst.probs.country$extreme_probs_hcst
  print("######### COUNTRY FCST PROBS COMPUTATION DONE ##############")
  
  print("######### FCST PROBS COMPUTATION DONE ##############")
  
  #----------------------------------------------------------
  # Saving calibrated forecast like .nc
  #----------------------------------------------------------
  
  
  outfile = paste0(path.outside,"/../",variable,"_country/",
                   variable,"-country-prob_",fcst.sdate,"01_",fcst.month,".nc")
  save_probs_country_file(variable,
                          tercile_probs_fcst.country,
                          extreme_probs_fcst.country,
                          fcst.sdate,
                          outfile,
                          fcst.times,
                          europe.countries.iso,
                          "seasonal")
  
  
  outfile = paste0(path.outside,"/../",variable,"_country/",
                   variable,"-country_",fcst.sdate,"01_",fcst.month, ".nc")
  save_ensemble_country_file(variable,
                             aperm(calibrated_fcst.country, c(2,1,3)),
                             fcst.sdate,
                             outfile,
                             fcst.times,
                             europe.countries.iso,
                             "seasonal")
  
}

cat("MEM USED: "); print(mem_used())

}

